==> stagexl-0.7.4.tar.gz: lib/src/display/DisplayObjectContainer.dart <==
part of stagexl;

abstract class DisplayObjectContainer extends InteractiveObject {
  
  final List<DisplayObject> _children = new List<DisplayObject>();
  bool _mouseChildren = true;
  bool _tabChildren = true;

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  int get numChildren => _children.length;

  bool get mouseChildren => _mouseChildren;
  bool get tabChildren => _tabChildren;

  void set mouseChildren(bool value) { _mouseChildren = value; }
  void set tabChildren(bool value) { _tabChildren = value; }

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  void addChild(DisplayObject child) {
    
    if (child.parent == this) {
      int index = _children.indexOf(child);
      _children.removeAt(index);
      _children.add(child);
    } else {
      addChildAt(child, _children.length);
    }
  }

  //-------------------------------------------------------------------------------------------------

  void addChildAt(DisplayObject child, int index) {
    
    if (index < 0 || index > _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    if (child == this)
      throw new ArgumentError("Error #2024: An object cannot be added as a child of itself.");

    if (child.parent == this) {
      
      _children.removeAt(_children.indexOf(child));

      if (index > _children.length)
        index --;

      _children.insert(index, child);
    
    } else {
      
      child.removeFromParent();

      child._setParent(this);
      _children.insert(index, child);

      child.dispatchEvent(new Event(Event.ADDED, true));

      if (this.stage != null)
        _dispatchEventOnChildren(child, new Event(Event.ADDED_TO_STAGE));
    }
  }

  //-------------------------------------------------------------------------------------------------

  void removeChild(DisplayObject child) {
    
    int childIndex = _children.indexOf(child);

    if (childIndex == -1)
      throw new ArgumentError("Error #2025: The supplied DisplayObject must be a child of the caller.");

    removeChildAt(childIndex);
  }

  //-------------------------------------------------------------------------------------------------

  void removeChildAt(int index) {
    
    if (index < 0 || index >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    DisplayObject child = _children[index];

    child.dispatchEvent(new Event(Event.REMOVED, true));

    if (this.stage != null)
      _dispatchEventOnChildren(child, new Event(Event.REMOVED_FROM_STAGE));

    child._setParent(null);
    _children.removeAt(index);
  }
  
  //-------------------------------------------------------------------------------------------------

  void removeChildren([int beginIndex, int endIndex]) {
    
    if (?beginIndex == false) beginIndex = 0;
    if (?endIndex == false) endIndex = _children.length - 1;
      
    if (beginIndex < 0 || beginIndex >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    if (endIndex < 0 || endIndex >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");
    
    for(int i = beginIndex; i <= endIndex; i++) {
      if (beginIndex >= _children.length) break;
      removeChildAt(beginIndex);
    }
  }

  //-------------------------------------------------------------------------------------------------

  dynamic getChildAt(int index) {
    
    if (index < 0 || index >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    return _children[index];
  }

  //-------------------------------------------------------------------------------------------------

  dynamic getChildByName(String name) {

    for(int i = 0; i < _children.length; i++)
      if (_children[i].name == name)
        return _children[i];

      return null;
  }

  //-------------------------------------------------------------------------------------------------

  int getChildIndex(DisplayObject child) {
      return _children.indexOf(child);
  }

  //-------------------------------------------------------------------------------------------------

  void setChildIndex(DisplayObject child, int index) {
    
    if (index < 0 || index >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    int oldIndex = getChildIndex(child);

    if (oldIndex == -1)
      throw new ArgumentError("Error #2025: The supplied DisplayObject must be a child of the caller.");

    _children.removeAt(oldIndex);
    _children.insert(index, child);
  }

  //-------------------------------------------------------------------------------------------------

  void swapChildren(DisplayObject child1, DisplayObject child2) {
    
      int index1 = getChildIndex(child1);
      int index2 = getChildIndex(child2);

      if (index1 == -1 || index2 == -1)
        throw new ArgumentError("Error #2025: The supplied DisplayObject must be a child of the caller.");

      swapChildrenAt(index1, index2);
  }

  //-------------------------------------------------------------------------------------------------

  void swapChildrenAt(int index1, int index2) {
    
    DisplayObject child1 = getChildAt(index1);
    DisplayObject child2 = getChildAt(index2);
    _children[index1] = child2;
    _children[index2] = child1;
  }

  //-------------------------------------------------------------------------------------------------

  void sortChildren(Function compareFunction) {
    _children.sort(compareFunction);
  }

  //-------------------------------------------------------------------------------------------------

  bool contains(DisplayObject child) {
    
    for(; child != null; child = child._parent)
      if (child == this)
        return true;

    return false;
  }

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  Rectangle getBoundsTransformed(Matrix matrix, [Rectangle returnRectangle]) {
    
    if (returnRectangle == null)
      returnRectangle = new Rectangle.zero();

    if (_children.length == 0)
      return super.getBoundsTransformed(matrix, returnRectangle);

    num left = double.INFINITY;
    num top = double.INFINITY;
    num right = double.NEGATIVE_INFINITY;
    num bottom = double.NEGATIVE_INFINITY;

    int childrenLength = _children.length;

    for (int i = 0; i < _children.length; i++) {
      
      DisplayObject child = _children[i];

      _tmpMatrix.copyFromAndConcat(child._transformationMatrix, matrix);
      Rectangle rectangle = child.getBoundsTransformed(_tmpMatrix, returnRectangle);

      if (rectangle.left < left) left = rectangle.left;
      if (rectangle.top < top ) top = rectangle.top;
      if (rectangle.right > right) right = rectangle.right;
      if (rectangle.bottom > bottom) bottom = rectangle.bottom;
    }

    returnRectangle.x = left;
    returnRectangle.y = top;
    returnRectangle.width = right - left;
    returnRectangle.height = bottom - top;

    return returnRectangle;
  }

  //-------------------------------------------------------------------------------------------------

  DisplayObject hitTestInput(num localX, num localY) {
    
    DisplayObject hit = null;

    for (int i = _children.length - 1; i >= 0; i--) {
      
      DisplayObject child = _children[i];

      if (child.visible) {
        Matrix matrix = child._transformationMatrix;

        double deltaX = localX - matrix.tx;
        double deltaY = localY - matrix.ty;
        double childX = (matrix.d * deltaX - matrix.c * deltaY) / matrix.det;
        double childY = (matrix.a * deltaY - matrix.b * deltaX) / matrix.det;

        var displayObject = child.hitTestInput(childX, childY);

        if (displayObject != null) {
          if (displayObject is InteractiveObject)
            if (displayObject.mouseEnabled)
              return _mouseChildren ? displayObject : this;

          hit = this;
        }
      }
    }

    return hit;
  }

  //-------------------------------------------------------------------------------------------------

  void render(RenderState renderState) {
    
    for(int i = 0; i < _children.length; i++) {
      
      DisplayObject child = _children[i];

      if (child.visible)
        renderState.renderDisplayObject(child);
    }
  }

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  void _dispatchEventOnChildren(DisplayObject displayObject, Event event) {
    
    displayObject.dispatchEvent(event);

    if (displayObject is DisplayObjectContainer) {
      
      var children = new List<DisplayObject>.from(displayObject._children, growable:false);

      for(int i = 0; i < children.length; i++)
        _dispatchEventOnChildren(children[i], event);
    }
  }


}

==> stagexl-0.7.5.tar.gz: lib/src/display/DisplayObjectContainer.dart <==
part of stagexl;

abstract class DisplayObjectContainer extends InteractiveObject {
  
  final List<DisplayObject> _children = new List<DisplayObject>();
  bool _mouseChildren = true;
  bool _tabChildren = true;

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  int get numChildren => _children.length;

  bool get mouseChildren => _mouseChildren;
  bool get tabChildren => _tabChildren;

  void set mouseChildren(bool value) { _mouseChildren = value; }
  void set tabChildren(bool value) { _tabChildren = value; }

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  void addChild(DisplayObject child) {
    
    if (child.parent == this) {
      int index = _children.indexOf(child);
      _children.removeAt(index);
      _children.add(child);
    } else {
      addChildAt(child, _children.length);
    }
  }

  //-------------------------------------------------------------------------------------------------

  void addChildAt(DisplayObject child, int index) {
    
    if (index < 0 || index > _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    if (child == this)
      throw new ArgumentError("Error #2024: An object cannot be added as a child of itself.");

    if (child.parent == this) {
      
      _children.removeAt(_children.indexOf(child));

      if (index > _children.length)
        index --;

      _children.insert(index, child);
    
    } else {
      
      child.removeFromParent();

      child._setParent(this);
      _children.insert(index, child);

      child.dispatchEvent(new Event(Event.ADDED, true));

      if (this.stage != null)
        _dispatchEventDescendants(child, new Event(Event.ADDED_TO_STAGE));
    }
  }

  //-------------------------------------------------------------------------------------------------

  void removeChild(DisplayObject child) {
    
    int childIndex = _children.indexOf(child);

    if (childIndex == -1)
      throw new ArgumentError("Error #2025: The supplied DisplayObject must be a child of the caller.");

    removeChildAt(childIndex);
  }

  //-------------------------------------------------------------------------------------------------

  void removeChildAt(int index) {
    
    if (index < 0 || index >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    DisplayObject child = _children[index];

    child.dispatchEvent(new Event(Event.REMOVED, true));

    if (this.stage != null)
      _dispatchEventDescendants(child, new Event(Event.REMOVED_FROM_STAGE));

    child._setParent(null);
    _children.removeAt(index);
  }
  
  //-------------------------------------------------------------------------------------------------

  void removeChildren([int beginIndex, int endIndex]) {
    
    if (?beginIndex == false) beginIndex = 0;
    if (?endIndex == false) endIndex = _children.length - 1;
      
    if (beginIndex < 0 || beginIndex >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    if (endIndex < 0 || endIndex >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");
    
    for(int i = beginIndex; i <= endIndex; i++) {
      if (beginIndex >= _children.length) break;
      removeChildAt(beginIndex);
    }
  }

  //-------------------------------------------------------------------------------------------------

  dynamic getChildAt(int index) {
    
    if (index < 0 || index >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    return _children[index];
  }

  //-------------------------------------------------------------------------------------------------

  dynamic getChildByName(String name) {

    for(int i = 0; i < _children.length; i++)
      if (_children[i].name == name)
        return _children[i];

      return null;
  }

  //-------------------------------------------------------------------------------------------------

  int getChildIndex(DisplayObject child) {
      return _children.indexOf(child);
  }

  //-------------------------------------------------------------------------------------------------

  void setChildIndex(DisplayObject child, int index) {
    
    if (index < 0 || index >= _children.length)
      throw new ArgumentError("Error #2006: The supplied index is out of bounds.");

    int oldIndex = getChildIndex(child);

    if (oldIndex == -1)
      throw new ArgumentError("Error #2025: The supplied DisplayObject must be a child of the caller.");

    _children.removeAt(oldIndex);
    _children.insert(index, child);
  }

  //-------------------------------------------------------------------------------------------------

  void swapChildren(DisplayObject child1, DisplayObject child2) {
    
      int index1 = getChildIndex(child1);
      int index2 = getChildIndex(child2);

      if (index1 == -1 || index2 == -1)
        throw new ArgumentError("Error #2025: The supplied DisplayObject must be a child of the caller.");

      swapChildrenAt(index1, index2);
  }

  //-------------------------------------------------------------------------------------------------

  void swapChildrenAt(int index1, int index2) {
    
    DisplayObject child1 = getChildAt(index1);
    DisplayObject child2 = getChildAt(index2);
    _children[index1] = child2;
    _children[index2] = child1;
  }

  //-------------------------------------------------------------------------------------------------

  void sortChildren(Function compareFunction) {
    _children.sort(compareFunction);
  }

  //-------------------------------------------------------------------------------------------------

  bool contains(DisplayObject child) {
    
    for(; child != null; child = child._parent)
      if (child == this)
        return true;

    return false;
  }

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  Rectangle getBoundsTransformed(Matrix matrix, [Rectangle returnRectangle]) {
    
    if (returnRectangle == null)
      returnRectangle = new Rectangle.zero();

    if (_children.length == 0)
      return super.getBoundsTransformed(matrix, returnRectangle);

    num left = double.INFINITY;
    num top = double.INFINITY;
    num right = double.NEGATIVE_INFINITY;
    num bottom = double.NEGATIVE_INFINITY;

    int childrenLength = _children.length;

    for (int i = 0; i < _children.length; i++) {
      
      DisplayObject child = _children[i];

      _tmpMatrix.copyFromAndConcat(child._transformationMatrix, matrix);
      Rectangle rectangle = child.getBoundsTransformed(_tmpMatrix, returnRectangle);

      if (rectangle.left < left) left = rectangle.left;
      if (rectangle.top < top ) top = rectangle.top;
      if (rectangle.right > right) right = rectangle.right;
      if (rectangle.bottom > bottom) bottom = rectangle.bottom;
    }

    returnRectangle.x = left;
    returnRectangle.y = top;
    returnRectangle.width = right - left;
    returnRectangle.height = bottom - top;

    return returnRectangle;
  }

  //-------------------------------------------------------------------------------------------------

  DisplayObject hitTestInput(num localX, num localY) {
    
    DisplayObject hit = null;

    for (int i = _children.length - 1; i >= 0; i--) {
      
      DisplayObject child = _children[i];

      if (child.visible) {
        Matrix matrix = child._transformationMatrix;

        double deltaX = localX - matrix.tx;
        double deltaY = localY - matrix.ty;
        double childX = (matrix.d * deltaX - matrix.c * deltaY) / matrix.det;
        double childY = (matrix.a * deltaY - matrix.b * deltaX) / matrix.det;

        var displayObject = child.hitTestInput(childX, childY);

        if (displayObject != null) {
          if (displayObject is InteractiveObject)
            if (displayObject.mouseEnabled)
              return _mouseChildren ? displayObject : this;

          hit = this;
        }
      }
    }

    return hit;
  }

  //-------------------------------------------------------------------------------------------------

  void render(RenderState renderState) {
    
    for(int i = 0; i < _children.length; i++) {
      
      DisplayObject child = _children[i];

      if (!child._off && child.visible)
        renderState.renderDisplayObject(child);
    }
  }

  //-------------------------------------------------------------------------------------------------
  //-------------------------------------------------------------------------------------------------

  _collectDescendants(DisplayObject displayObject, List<DisplayObject> descendants) {
    
    descendants.add(displayObject);
    
    if (displayObject is DisplayObjectContainer) {
      
      var displayObjectContainer = displayObject as DisplayObjectContainer;
      var children = displayObjectContainer._children;
    
      for(int i = 0; i < children.length; i++) {
        _collectDescendants(children[i], descendants);
      }
    }
  }
  
  _dispatchEventDescendants(DisplayObject displayObject, Event event) {

    var descendants = _displayObjectListPool.pop() as List<DisplayObject>;
    _collectDescendants(displayObject, descendants);

    for(int i = 0; i < descendants.length; i++) {
      descendants[i].dispatchEvent(event);
    }
    
    descendants.clear();
    _displayObjectListPool.push(descendants);
  }


}
